const CLASS_OPEN = 'qor-modal-open',
  ARIA_HIDDEN = 'aria-hidden';
/* On form cancel */
function onCancel() {
  let paths = window.location.pathname.split('/').filter((path) => path);
  let backURL = window.location.origin + '/' + paths[0] + '/' + paths[1];

  /* If it has opened on a new page, needs to go back with page reload otherwise without */
  if ($('.qor-slideout').hasClass('is-slided')) {
    window.history.pushState({}, null, backURL);
  } else {
    window.location.href = backURL;
  }
}

function CodeReferenceOnChange(inputId, value, crType) {
  const codeRefContainer = document.getElementById(`${inputId}_container`);
  codeRefContainer.style.display = 'flex';
  const codeRefEl = document.getElementById(inputId);
  let crValue;
  switch (crType) {
    case 'content':
      crValue = `safe .blocks.${value}.Content`;
      break;
    case 'image':
      crValue = `.blocks.${value}.Image.URL`;
      break;
    default:
      crValue = `.blocks.${value}.Content`;
      break;
  }
  codeRefEl.value = value == '' ? '' : `{{ ${crValue} }}`;
}

function copyToClipboard(elemId) {
  const codeRefEl = document.getElementById(elemId);
  navigator.clipboard.writeText(codeRefEl.value);
}

function hideHelpSection() {
  let overviewToggleButton = document.querySelector('.toggle-help-section');
  let overviewToggleSection = document.querySelector('.help-section');

  overviewToggleButton.style.display = 'none';
  overviewToggleSection.style.display = 'none';
  localStorage.setItem('hideHelpSection', 'hidden');
}

document.addEventListener(
  'DOMContentLoaded',
  function () {
    let overviewToggleButton = document.querySelector('.toggle-help-section');
    let overviewToggleSection = document.querySelector('.help-section');

    if (localStorage.getItem('hideHelpSection') === 'hidden' && overviewToggleButton) {
      overviewToggleButton.style.display = 'none';
      overviewToggleSection.style.display = 'none';
    }
  },
  false
);

function revealValue(elemId, emelValueId) {
  const codeRefEl = document.getElementById(elemId);
  const codeRefElValue = document.getElementById(emelValueId);
  console.log(elemId);
  console.log(emelValueId);
  if (codeRefEl.value === codeRefElValue.value) codeRefEl.value = '******';
  else codeRefEl.value = codeRefElValue.value;
}

function actionButtonToggleModal(id) {
  var modal = document.getElementById(id);
  if (modal.classList.contains('hidden')) {
    modal.classList.remove('hidden');
    modal.classList.add('shown');
    $(document).find('body').addClass(CLASS_OPEN);
    $(modal).attr(ARIA_HIDDEN, false);
  } else {
    modal.classList.add('hidden');
    modal.classList.remove('shown');
    $(document).find('body').removeClass(CLASS_OPEN);
    $(modal).attr(ARIA_HIDDEN, true);
  }
}

function actionButtonExecute(reloadPage, actionbuttonModalId, url, method, data, datatype) {
  $.ajax(url, {
    xhrFields: {
      withCredentials: true,
    },
    method,
    data,
    dataType: datatype || 'json',
    beforeSend: function (xhr) {
      console.log('beforeSend');
    },
    success: function (data) {
      console.log('success.data:', data);
    },
    error: function (err) {
      if (err.status == 200) {
        return;
      }
    },
    complete: function (response) {
      if (reloadPage) {
        window.location.reload();
      }
    },
  });
}

function actionHandler(name, menu_id, id, move) {
  let path = name.split(",")
  let url = `/admin/${path[0]}/${menu_id}/${path[1]}/${id}/${move}`
  actionButtonExecute(false,null,url,"PUT")
}

(function(factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD. Register as anonymous module.
        define(['jquery'], factory);
    } else if (typeof exports === 'object') {
        // Node / CommonJS
        factory(require('jquery'));
    } else {
        // Browser globals.
        factory(jQuery);
    }
})(function($) {
    'use strict';

    let NAMESPACE = 'qor.collection.sortable',
        EVENT_ENABLE = 'enable.' + NAMESPACE,
        EVENT_DISABLE = 'disable.' + NAMESPACE,
        EVENT_CLICK = 'click.' + NAMESPACE,
        CLASS_ITEM = '.qor-sortable__item',
        CLASS_CHILDREN_ITEM = '> .qor-field__block > .qor-sortable__item',
        CLASS_BUTTON_CHANGE = '>.qor-sortable__button > .qor-sortable__button-change',
        CLASS_BUTTON_DONE = '>.qor-sortable__button > .qor-sortable__button-done',
        CLASS_BUTTON_ADD = '> .qor-field__block > .qor-sortable__button-add',
        CLASS_BUTTON_DELETE = '> .qor-sortable__button-delete',
        CLASS_BUTTON_MOVE = '>.qor-field__block > .qor-sortable__item > .qor-sortable__change .qor-sortable__button-move',
        CLASS_ACTION = '> .qor-sortable__change > .qor-sortable__action',
        CLASS_ACTION_POSITION = '.qor-sortable__action-position',
        CLASS_ITEM_FILTER = '.is-delete,.qor-fieldset--new',
        IS_LOADED = 'sortable-collection-loaded';

    function moveArrPos(arr, from, name) {
      switch(name){
        case "move_up":
          if (from > 0) {
            let toVal = arr[from-1]
            let fromVal = arr[from]
            arr[from] = toVal
            arr[from-1] = fromVal
          }
          break
        case "move_down":
          if (from < arr.length) {
            let toVal = arr[from+1]
            let fromVal = arr[from]
            arr[from] = toVal
            arr[from+1] = fromVal
          }
          break
      }
      return arr;
    }

    function QorCollectionSortable(element, options) {
        this.$element = $(element);
        this.options = $.extend({}, QorCollectionSortable.DEFAULTS, $.isPlainObject(options) && options);
        this.init();
    }

    QorCollectionSortable.prototype = {
        constructor: QorCollectionSortable,

        init: function() {
            this.bind();
            this.initItemOrder();
        },

        bind: function() {
            this.$element
                .on(EVENT_CLICK, CLASS_BUTTON_MOVE, this.moveItem.bind(this))
                .on(EVENT_CLICK, CLASS_BUTTON_DONE, this.finishMoveItem.bind(this))
                .on(EVENT_CLICK, CLASS_BUTTON_CHANGE, this.startMoveItem.bind(this));

            $(document).on('added.qor.replicator.sortableCollection', this.resetOrderArr.bind(this));
        },

        unbind: function() {
            this.$element
                .off(EVENT_CLICK, CLASS_BUTTON_MOVE)
                .off(EVENT_CLICK, CLASS_BUTTON_DONE)
                .off(EVENT_CLICK, CLASS_BUTTON_CHANGE);
            $(document).off('added.qor.replicator.sortableCollection');
        },

        initItemOrder: function(resetResource) {
            let $item = this.$element.find(CLASS_CHILDREN_ITEM).not(CLASS_ITEM_FILTER);

            // return false if no item
            if (!$item.length) {
                return;
            }

            this.$item = $item;

            let $select = $item.find(CLASS_ACTION).find(CLASS_ACTION_POSITION),
                orderData = {},
                orderArr = [],
                itemTotal = $item.length,
                template = $item.first().html(),
                fullResourceName,
                resourceNamePrefix;

            if ($select.length) {
                $select.remove();
            }

            if (!resourceNamePrefix) {
                fullResourceName = template.match(/(\w+)\="(\S*\[\d+\]\S*)"/); // get results : [attribute, name, value]
                if (fullResourceName.length) {
                    fullResourceName = fullResourceName[2];
                    resourceNamePrefix = fullResourceName.match(/^(\S*)\[(\d+)\]([^\[\]]*)$/); // get results : [input, prefix, index, suffix]

                    if (resourceNamePrefix.length) {
                        resourceNamePrefix = resourceNamePrefix[1];
                    }
                }
            }

            $item.each(function() {
                let $this = $(this),
                    $action = $this.find(CLASS_ACTION),
                    itemIndex = parseInt($this.attr('order-index'));

                orderData.itemTotal = itemTotal;
                orderData.itemIndex = itemIndex;
                orderArr.push(itemIndex);

                /* $action.prepend('<select class="qor-sortable__action-position"></select>'); */

                for (let i = 1; i <= itemTotal; i++) {
                    let renderData = {},
                        isSelected;

                    itemIndex + 1 == i ? (isSelected = true) : (isSelected = false);
                    renderData = $.extend({}, orderData, {selectorPosition: i, isSelected: isSelected});

                    $action.find('select').append(window.Mustache.render(QorCollectionSortable.OPTION_HTML, renderData));
                }

                // reset form resource name prop
                if (resetResource) {
                    let resourceName,
                        newPosition,
                        resourceNameEnd,
                        hasPrefixPosition = /\[\d+\]/.test(resourceNamePrefix),
                        $resource = $(this).find('[name^="' + resourceNamePrefix + '"]');

                    if (!$resource.length) {
                        return;
                    }

                    $resource.each(function() {
                        resourceName = $(this).prop('name');
                        resourceNameEnd = resourceName.match(/\.\w+$/);
                        newPosition = `[${itemIndex}]`;

                        if (hasPrefixPosition) {
                            // fullResourceName = QorResource.SerializableMeta.Sections[0].Items[0(this number is the position)].Name
                            // resourceNamePrefix = QorResource.SerializableMeta.Sections[0].Items
                            // change the last [0].Name to real position
                            resourceName = resourceName.replace(/\[\d+\]\.\w+$/, `${newPosition}${resourceNameEnd}`);
                        } else {
                            resourceName = resourceName.replace(/\[\d+\]/, newPosition);
                        }

                        $(this).prop('name', resourceName);
                    });
                }

                $this.data(orderData);
            });
            this.itemOrderArr = this.itemOrderArr || orderArr;
        },

        resetOrderArr: function(e, $item) {
            let orderIndex = parseInt($item.attr('order-index'));
            if (!orderIndex) {
                return;
            }

            this.itemOrderArr = this.itemOrderArr || [];
            this.itemOrderArr.push(orderIndex);
            this.$item = this.$element.find(CLASS_CHILDREN_ITEM).not(CLASS_ITEM_FILTER);
        },

        moveItem: function(e) {
            let $current = $(e.target).closest(CLASS_ITEM),
                currentPosition = $current.data().itemIndex,
                targetPosition = parseInt($current.find(CLASS_ACTION_POSITION).val()) - 1,
                $item = this.$item,
                name = $(e.target).attr('name'),
                newPosArr = [];
            /* if (targetPosition == currentPosition) {
                return;
            } */

            newPosArr = moveArrPos(this.itemOrderArr, currentPosition, name);

            newPosArr.forEach(function(arr) {
                let index = newPosArr.indexOf(arr);

                $item
                    .filter(`[order-item="item_${arr}"]`)
                    .attr('order-index', index)
                    .css('order', index);
            });
            this.itemOrderArr = newPosArr;
            this.initItemOrder(true);
        },

        finishMoveItem: function(e) {
            let $target = $(e.target),
                $element = this.$element,
                $item = $element.find(CLASS_CHILDREN_ITEM).not(CLASS_ITEM_FILTER),
                $addButton = $element.find(CLASS_BUTTON_ADD),
                $deleteButton = $item.find(CLASS_BUTTON_DELETE),
                $actionButtons = $item.find(CLASS_ACTION);

            $target.hide();
            $actionButtons.hide();

            $element.find(CLASS_BUTTON_CHANGE).show();
            $addButton.show();
            $deleteButton.show();
        },

        startMoveItem: function(e) {
            let $target = $(e.target),
                $element = this.$element,
                $item = $element.find(CLASS_CHILDREN_ITEM).not(CLASS_ITEM_FILTER),
                $addButton = $element.find(CLASS_BUTTON_ADD),
                $deleteButton = $item.find(CLASS_BUTTON_DELETE),
                $actionButtons = $item.find(CLASS_ACTION);

            if (!$item.length) {
                return false;
            }

            $target.hide();

            $element.find(CLASS_BUTTON_DONE).show();
            $actionButtons.show();
            $addButton.hide();
            $deleteButton.hide();
            this.initItemOrder();
        },

        destroy: function() {
            this.unbind();
            this.$element.removeData(NAMESPACE);
        }
    };

    QorCollectionSortable.DEFAULTS = {};

    QorCollectionSortable.OPTION_HTML = '<option value="[[selectorPosition]]" [[#isSelected]]selected[[/isSelected]]>[[selectorPosition]] of [[itemTotal]]</option>';

    QorCollectionSortable.plugin = function(options) {
        return this.each(function() {
            var $this = $(this);
            var data = $this.data(NAMESPACE);
            var fn;

            if (!data) {
                if (/destroy/.test(options)) {
                    return;
                }

                $this.data(NAMESPACE, (data = new QorCollectionSortable(this, options)));
            }

            if (typeof options === 'string' && $.isFunction((fn = data[options]))) {
                fn.apply(data);
            }
        });
    };

    $(function() {
        var selector = '[data-toggle="qor.collection.sortable"]';

        if ($('body').data(IS_LOADED)) {
            return;
        }
        $('body').data(IS_LOADED, true);

        $(document)
            .on(EVENT_DISABLE, function(e) {
                QorCollectionSortable.plugin.call($(selector, e.target), 'destroy');
            })
            .on(EVENT_ENABLE, function(e) {
                QorCollectionSortable.plugin.call($(selector, e.target));
            })
            .triggerHandler(EVENT_ENABLE);
    });

    return QorCollectionSortable;
});

function toggleRedactor(showClass, hideClass) {
  $(showClass).show();
  $(hideClass).hide();
  if (showClass == ".markdow-editor-wrapper") {
    let htmlCol = document.querySelectorAll(showClass);
    htmlCol.forEach(e => {
      let editors = e.getElementsByClassName('EasyMDEContainer');
      if (editors.length >=2) {
        editors[editors.length-1].parentNode.removeChild(editors[editors.length-1])
      }
    });
  }
}

function removeTag(el) {
  let element = el.parentNode;
  element.remove();
  //TODO: remove from input value
  let input = document.querySelector('.tags-input-value');
  let value = element.innerText.substring(1);
  input.value = input.value.split(",").filter(e => e !== value);
  console.log(input)
}
function waitForElm(selector) {
    return new Promise(resolve => {
      if (document.querySelector(selector)) {
          return resolve(document.querySelector(selector));
      }

      const observer = new MutationObserver(mutations => {
          if (document.querySelector(selector)) {
              resolve(document.querySelector(selector));
              observer.disconnect();
          }
      });

      observer.observe(document.body, {
          childList: true,
          subtree: true
      });
    });
};

function autocomplete(inp, arr) {
  var currentFocus;
  inp.addEventListener("input", function(e) {
      var a, b, i, val = this.value;
      closeAllLists();
      if (!val) { return false;}
      currentFocus = -1;
      a = document.createElement("DIV");
      a.setAttribute("id", this.id + "autocomplete-list");
      a.setAttribute("class", "autocomplete-items");
      this.parentNode.appendChild(a);
      for (i = 0; i < arr.length; i++) {
        if (arr[i].Name.substr(0, val.length).toLowerCase() == val.toLowerCase()) {
          b = document.createElement("DIV");
          b.innerHTML = "<strong>" + arr[i].Name.substr(0, val.length) + "</strong>";
          b.innerHTML += arr[i].Name.substr(val.length);
          b.innerHTML += "<input type='hidden' value='" + arr[i].Name + "'>";
          b.style.cursor = "pointer";
          b.classList.add("autocomplete-tag-item");
          b.addEventListener("click", function(e) {
            let value = this.getElementsByTagName("input")[0].value
            inp.value = "";
            createTag(value);
            closeAllLists();
          });
          a.appendChild(b);
        }
      }
  });
  inp.addEventListener("keydown", function(e) {
      var x = document.getElementById(this.id + "autocomplete-list");
      if (x) x = x.getElementsByTagName("div");
      if (e.keyCode == 40) {
        currentFocus++;
        addActive(x);
      } else if (e.keyCode == 38) { 
        currentFocus--;
        addActive(x);
      } else if (e.keyCode == 13) {
        e.preventDefault();
        if (currentFocus > -1) {
          if (x) x[currentFocus].click();
        }
        let value = e.target.value.toLowerCase().trim();
        if (value !== "") {
          createTag(value);
          inp.value = "";
        }
      }
  });
  function addActive(x) {
    if (!x) return false;
    removeActive(x);
    if (currentFocus >= x.length) currentFocus = 0;
    if (currentFocus < 0) currentFocus = (x.length - 1);
    x[currentFocus].classList.add("autocomplete-active");
    x[currentFocus].classList.remove("autocomplete-tag-item");
  }
  function removeActive(x) {
    for (var i = 0; i < x.length; i++) {
      x[i].classList.remove("autocomplete-active");
      x[i].classList.add("autocomplete-tag-item");
    }
  }
  function closeAllLists(elmnt) {
    var x = document.getElementsByClassName("autocomplete-items");
    for (var i = 0; i < x.length; i++) {
      if (elmnt != x[i] && elmnt != inp) {
      x[i].parentNode.removeChild(x[i]);
    }
  }
  }
  document.addEventListener("click", function (e) {
      closeAllLists(e.target);
  });
}
function createTag(value) {
  let input = document.querySelector('.tags-input-value');
  if (!input.value.split(",").includes(value)){
    let parent = document.querySelector('.tags-selection--multiple');
    let tagsSelectMany = document.querySelector('.tags-select-many');
    let newTag = document.createRange().createContextualFragment(`<div class="select2-selection__choice"><span class="select2-selection__choice__remove" onclick="removeTag(this)" role="presentation">×</span>${value}</div>`);
    parent.insertBefore(newTag,tagsSelectMany);
    input.value += `,${value}`
  }
} 
waitForElm('.tags-select-many').then((elm) => {
  $.ajax("/admin/tags.json", {
    method: 'get',
    dataType: 'json',
    success: function(res) {
      autocomplete(elm,res)
    },
    error: function(xhr, textStatus, errorThrown) {
        if (xhr.status === 422) {
            window.alert(xhr.responseText);
        } else {
            window.alert([textStatus, errorThrown].join(': '));
        }
        location.reload();
    }
  })
});
